@@ -1,9 +1,10 @@
 #' Merge two ffdf by common columns, or do other versions of database join operations. 
 #'
-#' Merge two ffdf by common columns, or do other versions of database join operations. \cr
-#' This method is similar as \code{merge} in the base package but only allows inner and left outer joins.\cr
-#' Mark that joining is done based on ffmatch or ffdfmatch, meaning that only the first element in \code{y} will be added to \code{x} and ffdfmatch works on
-#' \code{link[base]{paste}}-ing together a key. So this might not be suited if your key contains columns of vmode double.\cr
+#' Merge two ffdf by common columns, or do other versions of database join operations.
+#' This method is similar to \code{\link[base]{merge}} in the \code{base} package but only allows inner and left outer joins.
+#' Note that joining is done based on \code{\link{ffmatch}} or \code{\link{ffdfmatch}}: only the first element
+#' in \code{y} will be added to \code{x}; and since \code{\link{ffdfmatch}} works by \code{\link[base]{paste}}-ing together a key, 
+#' this might not be suited if your key contains columns of vmode double.
 #'
 #' If a left outer join is performed and no matching record in x is found in y, columns with vmodes 'boolean', 'quad', 'nibble', 'ubyte', 'ushort' are
 #' coerced respectively to vmode 'logical', 'byte', 'byte', 'short', 'integer' to allow NA values.
@@ -36,7 +37,13 @@ merge.ffdf <- function(x, y, by = intersect(names(x), names(y)), by.x = by, by.y
   if((all.x == TRUE & all.y == TRUE) | (all.y == TRUE & all.x == TRUE)){
   	stop("merge.ffdf only allows inner joins")
   }
-  ## First find the first match where the left hand side ffdf is in the right hand side ffdf
+  if (!all(by.x %in% names(x))){
+    stop("'",by.x[!by.x %in% names(x)], "' is missing from x")
+  }
+	if (!all(by.y %in% names(y))){
+	  stop("'", by.y[!by.y %in% names(y)], "' is missing from y")
+	}
+	## First find the first match where the left hand side ffdf is in the right hand side ffdf
   if(length(by.x) == 1){
   	matchidx <- eval(ffmatch(x[[by.x]], y[[by.y]], trace = trace))
   }else{
@@ -57,9 +64,11 @@ merge.ffdf <- function(x, y, by = intersect(names(x), names(y)), by.x = by, by.y
   		jointype <- "inner"
   	}
   }
-  measures <- colnames(y)[!colnames(y) %in% by.y]
-  newnamesmeasures <- ifelse(measures %in% colnames(x), paste(measures, suffixes[2], sep=""), measures)
-  if(length(measures) == 0){
+  xvars <- colnames(x)
+  newxvars <- ifelse(!xvars %in% by.x & xvars %in% colnames(y), paste(xvars, suffixes[1], sep=""), xvars)
+  yvars <- colnames(y)[!colnames(y) %in% by.y]
+  newyvars <- ifelse(yvars %in% colnames(x), paste(yvars, suffixes[2], sep=""), yvars)
+  if(length(yvars) == 0){
   	stop("merge.ffdf requires at least 1 column in y not in by.y")
   }
   ## JOIN  
@@ -69,12 +78,12 @@ merge.ffdf <- function(x, y, by = intersect(names(x), names(y)), by.x = by, by.y
   	if(trace) {
     	message(sprintf("%s, found match indexes, now starting to add y to x", Sys.time()))
   	}	
-  	res <- ffbaseffdfindexget(y[measures], index=matchidx[updatepositions], ...)
-  	colnames(res) <- newnamesmeasures
+  	res <- ffbaseffdfindexget(y[yvars], index=matchidx[updatepositions], ...)
   	## cbind it to the existing x ffdf if there is a match
   	x <- ffbaseffdfindexget(x, index=updatepositions, ...)
-  	for(i in 1:length(measures)){
-  		x[[newnamesmeasures[i]]] <- res[[measures[i]]]
+  	colnames(x) <- newxvars
+  	for(i in 1:length(yvars)){
+  		x[[newyvars[i]]] <- res[[yvars[i]]]
   	}	  	 
   }else if(jointype == "left"){
   	##
@@ -82,23 +91,23 @@ merge.ffdf <- function(x, y, by = intersect(names(x), names(y)), by.x = by, by.y
   	if(trace){
     	message(sprintf("%s, found match indexes, now starting to add y to x and coercing if needed", Sys.time()))
   	}  	
-  	tocoerce <- coerce_to_allowNA(vmode(y[measures]))  	
+  	tocoerce <- coerce_to_allowNA(vmode(y[yvars]))  	
   	## First set everything to NA  	
-  	for(i in 1:length(measures)){
-  		x[[newnamesmeasures[i]]] <- clone(y[[measures[i]]], initdata=NA, length = nrow(x), vmode = tocoerce$coerceto[[measures[i]]])
+  	for(i in 1:length(yvars)){
+  		x[[newyvars[i]]] <- clone(y[[yvars[i]]], initdata=NA, length = nrow(x), vmode = tocoerce$coerceto[[yvars[i]]])
   	} 
   	## Next coerce the right hand side if needed to allow NA values
-  	tocoerce.measures <- measures[tocoerce$x != tocoerce$coerceto]
-  	if(length(tocoerce.measures) > 0){
-  		warning(paste("coercing column ", paste(tocoerce.measures, collapse=", "), " to a higher vmode to allow NA's"), sep="")
-  		for(measure in tocoerce.measures){
+  	tocoerce.yvars <- yvars[tocoerce$x != tocoerce$coerceto]
+  	if(length(tocoerce.yvars) > 0){
+  		warning(paste("coercing column ", paste(tocoerce.yvars, collapse=", "), " to a higher vmode to allow NA's"), sep="")
+  		for(measure in tocoerce.yvars){
  				y[[measure]] <- clone(y[[measure]], vmode = tocoerce$coerceto[[measure]])
   		}
   	} 		
   	## Next update the existing x ffdf with the joined value if there is a match  	
   	if(length(updatepositions) > 0){
 	  	matchidx.nonmissing <- matchidx[updatepositions]
-  		x[updatepositions, newnamesmeasures] <- ffbaseffdfindexget(y[measures], index=matchidx.nonmissing)
+  		x[updatepositions, newyvars] <- ffbaseffdfindexget(y[yvars], index=matchidx.nonmissing)
   	}  	
   }    
 	x
